/*
* CLIF is a Load Injection Framework
* Copyright (C) 2008-2009,2012-2013 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.storage.lib.filestorage;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;

import org.ow2.clif.console.lib.TestPlanReader;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.util.ExecutionContext;
import org.ow2.clif.util.FileUtil;

public abstract class FileStorageCommons
{
	static public final String COMMENT_PREFIX = "#";
	static public final String REPORT_DIR_PROP = "clif.filestorage.dir";
	static public final String REPORT_DIR_DEFAULT = "report";
	static public final String CSV_EXTENSION = ".csv";
	static public final String CLASSNAME_EXTENSION = ".classname";
	static public final String JVMPROPS_FILENAME = "server.prop";
	static public final String CLEAN_POLICY_PROP = "clif.filestorage.clean";
	static public final String CLEAN_POLICY_NONE = "none";
	static public final String CLEAN_POLICY_AUTO = "auto";


	static String getTestDirbase()
	throws ClifException
	{
		return getTestDirbase(null, true);
	}


	static synchronized String getTestDirbase(String dirname, boolean create)
	throws ClifException
	{
		String testDirbase = null;
		boolean testDirbaseFailed;
		if (dirname == null)
		{
			 dirname = System.getProperty(REPORT_DIR_PROP, REPORT_DIR_DEFAULT);
		}
		File dir = new File(dirname);
		if (dir.isAbsolute())
		{
			testDirbase = dir.getAbsolutePath();
		}
		else if (ExecutionContext.WORKSPACE_PATH != null)
		{
			testDirbase = ExecutionContext.WORKSPACE_PATH + File.separator + dir;
		}
		else
		{
			testDirbase = ExecutionContext.getBaseDir() + dir;
		}
		File reportDir = new File(testDirbase);
		if (create && ! reportDir.canWrite() && ! reportDir.mkdirs())
		{
			testDirbaseFailed = ! reportDir.canWrite();
		}
		else
		{
			testDirbaseFailed = ! reportDir.canRead();
		}
		if (testDirbaseFailed)			
		{
			//throw new ClifException("Storage error: no access to directory " + testDirbase);			
			// If not accessible use tmp			
			File f = new File(System.getProperty("java.io.tmpdir"), dirname);
			if (f.exists()) {
				return f.getAbsolutePath();	
			}
			if ( !f.mkdir() ) {
				throw new ClifException("Since no access to storage directory " + testDirbase + " unable to create testDirbase " + f);
			}					
			f.deleteOnExit();
			return f.getAbsolutePath();			
		}
		else
		{
			return testDirbase;
		}
	}

	/**
	 * Creates a directory for a new test execution:
	 * <ul>
	 *   <li>either just the root directory for all measurement files
	 *   generated by this test</li>
	 *   <li>or also the sub-directory for a given blade's measurement files</li>
	 * </ul>
	 * Then, the cleaning policy for measurement files is applied, possibly discarding
	 * measurement files from previous tests. 
	 * @param testId the new test's name
	 * @param bladeId the blade identifier. When null, just the root directory
	 * for all output files of the new test is created.
	 * @return the created directory
	 * @throws ClifException the directory could not be created. See messages and possible
	 * embedded exceptions for troubleshooting.
	 */
	static synchronized File newTestDir(Serializable testId, String bladeId)
	throws ClifException
	{
		String testDirname = FileStorageCommons.getTestDirbase() + File.separator + testId;
		if (bladeId != null)
		{
			testDirname += File.separator + bladeId;
		}
		File testDir = new File(testDirname);
		try
		{
			int i = 0;
			while (testDir.exists())
			{
				testDirname = FileStorageCommons.getTestDirbase() + File.separator + testId + "-" + ++i;
				if (bladeId != null)
				{
					testDirname += File.separator + bladeId;
				}
				testDir = new File(testDirname);
			}
			if (! testDir.exists() && ! testDir.mkdirs())
			{
				throw new IOException("Could not create " + testDirname + " directory.");
			}
		}
		catch (Exception e)
		{
			throw new ClifException("Can't create directory for test run " + testId, e);
		}
		// discard previous test execution measurement files according to the cleaning policy
		synchronized (FileStorageCommons.class)
		{
			File keepDir = testDir;
			if (bladeId != null)
			{
				keepDir = testDir.getParentFile();
			}
			// "auto" policy: discard all tests' output files but the new test's files
			if (
				System.getProperty(
					FileStorageCommons.CLEAN_POLICY_PROP,
					FileStorageCommons.CLEAN_POLICY_NONE
				).equalsIgnoreCase(FileStorageCommons.CLEAN_POLICY_AUTO))
			{
				for (File f : new File(FileStorageCommons.getTestDirbase()).listFiles())
				{
					if (
						! f.equals(keepDir)
						&& ! f.equals(
							new File(keepDir.getPath() + TestPlanReader.FILE_EXT)))
					{
						FileUtil.delete(f);
					}
				}
			}
		}
		return testDir;
	}
}
